'use strict';
/* General Scripts */
var ISML = require('dw/template/ISML');
var guard = require('~/cartridge/scripts/guard');

/* Logger */
var logger = dw.system.Logger.getLogger('nexi.xpay.bm');

/* XPay scripts from int_nexixpay and bm_nexixpay cartridges */
var XPayUtils = require("~/../int_nexixpay/cartridge/scripts/util/XPayUtils");
var BackofficeSVC = require("~/cartridge/scripts/svc/XPayBackoffice");

/* Internal constants */
const MODE_REIMBURSE = 'Reimburse';
const EDIT_AMOUNT = 'EditAmount';
const MAX_QUERY_RESULTS = 100;

/**
 * Renders the NexiPayTransactionSearch template with all related args
 * @param args
 * @returns
 */
function renderSearch(args) {

	args = args || {};

    return ISML.renderTemplate('NexiPayTransactionSearch', {
        ContinueURL     : dw.web.URLUtils.https('NexiPayTransactionMng-HandleForm'),
        CurrentForms    : session.forms,
        transactions 	: args.transactions || [],
        TOP_URL         : args.topUrl,
        MAIN_MENU_NAME  : args.mainMenuName,
        errorMsg 		: args.errorMsg || '',
        from			: args.from,
        to				: args.to,
        mode			: args.mode,
    });
}

/**
 * Renders the NexiPayTransactionDetails template with all related args
 * @param args
 * @returns
 */
function renderTransactionDetails(args){

	if(args.alias != null){
		var alias = args.alias;
	}else{
		var alias = request.httpParameterMap.alias.stringValue || session.custom.alias;
	}

	session.custom.alias = alias;

	args = args || {};

	return ISML.renderTemplate('NexiPayTransactionDetails', {
        ContinueURL     : dw.web.URLUtils.https('NexiPayTransactionMng-HandleForm'),
        CurrentForms    : session.forms,
		transaction 	: args.transaction,
		transactionId 	: args.transactionId,
		//autorizzato     : importoAutorizzato,
		//contabilizzato  : importoContabilizzato,
		//stornato        : importoStornato,

        TOP_URL         : args.topUrl,
        MAIN_MENU_NAME  : args.mainMenuName,
        TOP_URL2        : dw.web.URLUtils.https('NexiPayTransactionMng-Start', 'mainmenuname', args.mainMenuName, 'TopUrl', args.topUrl, 'mode', args.mode),
        errorMsg 		: args.errorMsg || '',
        successMsg		: args.successMsg || '',
        editAmount		: args.editAmount,
        editAmountLink	: dw.web.URLUtils.url('NexiPayTransactionMng-SelectTransaction', 
							'TransactionId', args.transactionId,
							'TopUrl', args.topUrl,
							'MainMenuName', args.mainMenuName,
							'EditMode', EDIT_AMOUNT,
							'mode', args.mode,
							'alias', alias),
		mode			: args.mode,
		alias           : alias
    });
}

/**
 * Validates the result of XPay services by checking the "mac" field
 * @param jsonResponse - XPay result object
 * @returns
 */
function validateResultMac(jsonResponse,alias){


	if(alias == null){alias = XPayUtils.XPAY_TERM_WEB_ALIAS}

	var terminal = XPayUtils.getTermByAlias(alias);

	var macCalculated = XPayUtils.sha1(
		'esito=' + jsonResponse.esito + 
		'idOperazione=' + jsonResponse.idOperazione + 
		'timeStamp=' + jsonResponse.timeStamp + 
		terminal.secretkey);
	
	if(jsonResponse.mac != macCalculated) {	

		throw { 
			errorMessage: dw.web.Resource.msg('search.macNotValidated', 'backoffice', null),
			error: '-',
		}
	}

	return jsonResponse;
}

/**
 * Handles the call for transactions list (see BackofficeSVC.callTransactionsList() for details on params)
 * UPDATE: this API has been replaced with a simple query on orders by transaction/order no.
 * @param args <Object>
 * @returns JSON result
 */
function getXPayTransactions(args) {

	try{
		var params = {
				transactionCode: args.transactionid,
				period: args.period,
				channel: args.channel,
				status: args.status,
		};
		var httpResult = BackofficeSVC.callTransactionsList(params);
		if(!httpResult.ok) {			
			throw httpResult;
		}
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callTransactionsList-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));
		return validateResultMac(jsonResponse);
	}
	catch(ex){
		logger.error('Eccezione: '+JSON.stringify(ex));
		return{
			esito: 'KO',
			errore:{
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}


/**
 * Handles the call for transaction details (see BackofficeSVCcallTransactionDetail() for details on params)
 * @param args <Object>
 * @returns JSON result
 */
function getXPayTransaction(transactionCode,alias){
	
	try{
		var params = {
				transactionCode: transactionCode,
				alias:alias
		};
		
		var httpResult = BackofficeSVC.callTransactionDetail(params);
		
		if(!httpResult.ok) {			
			throw httpResult;
		}
		
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callTransactionDetail-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));

		return validateResultMac(jsonResponse,alias);
	}
	catch(ex) {
		logger.error('Eccezione: '+JSON.stringify(ex));
		return{
			esito: 'KO',
			errore:{
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}

/**
 * Handles the call for reimburse (see BackofficeSVC.callReimburse() for details on params)
 * @param args <Object>
 * @returns JSON result
 */
function doXPayReimburse(args){
	try {
		var params = {
				transactionCode: args.transactionId,
				amount: args.amount,
				alias: args.alias
		};
		var httpResult = BackofficeSVC.callReimburse(params);
		if(!httpResult.ok) {			
			throw httpResult;
		}
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callReimburse-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));		
		return validateResultMac(jsonResponse,args.alias);
	}
	catch(ex) {
		logger.error('Eccezione: '+JSON.stringify(ex));
		return{
			esito: 'KO',
			errore:{
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}

/**
 * Handles the call for taking (see BackofficeSVC.callTake() for details on params)
 * @param args <Object>
 * @returns JSON result
 */
function doXPayCapture(args) {
	try {
		var params = {
				transactionCode: args.transactionId,
				amount: args.amount,
				alias: args.alias
		};
		var httpResult = BackofficeSVC.callTake(params);
		if(!httpResult.ok) {			
			throw { 
				errMsg: '['+httpResult.error+'] ' + httpResult.errorMessage
			}
		}
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callTake-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));

		return validateResultMac(jsonResponse,args.alias);
	}
	catch(ex) {
		logger.error('Eccezione: '+JSON.stringify(ex));
		return{
			esito: 'KO',
			errore: {
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}

/**
 * Query for a given order by the Nexi transaction id
 * @param transactionId <String>
 * @returns
 */ 
function getOrderByTransactionId(transactionId){	
	return dw.order.OrderMgr.searchOrder('custom.xpay_transactionCode = {1}', null, transactionId);
}

/**
 * Gets the order link on the business manager given its orderNo 
 * @param orderNo <String>
 * @returns
 */

/*
function getOrderLink(orderNo){
	return dw.web.URLUtils.url('ViewOrder_52-FindByNumber', 'OrderNo', orderNo);	
}
*/

/**
 * Transform a yyyy-mm-dd date string into dd/mm/yyyy 
 * @param val <String> yyyy-mm-dd
 * @returns <String> dd-mm-yyyy
 */
function getXPayDateFromString(val){	//@todo spostare su XPayUtils
	if(!val) return null;
	if(val.length !== 10) return null;
	var tmp = val.split('-');
	return tmp[2]+'/'+tmp[1]+'/'+tmp[0];
}

/**
 * read the a transaction js object saved in a form field used as a cache
 * @returns if available, return a transaction, return null otherwise 
 */
function getCachedTransaction() {
	return session.forms.bosearch.transaction_cache.value 
	           ? JSON.parse(session.forms.bosearch.transaction_cache.value) 
			   : null; 
}

/**
 * Gets all the transactions controller's logic
 * @returns
 */
function getTransactions() {

	//used to cache transaction call to Nexi
	session.forms.bosearch.transaction_cache.value = null;
	
	var args = {};
	args.errorMsg = '';
	args.transactionid = session.forms.bosearch.transactionid.value;
	//args.from = getXPayDateFromString(session.forms.bosearch.fromdate.value);
	//args.to = getXPayDateFromString(session.forms.bosearch.todate.value);
	args.status = ['Autorizzato'];//, 'Negato', 'Annullato'];
	args.period = (args.from && args.to) ? (args.from+' - '+ args.to) : '';
	args.channel = 'All';
	args.topUrl = session.forms.bosearch.bc_root_link.value;
	args.mainMenuName = session.forms.bosearch.bc_root_name.value;
	args.mode = session.forms.bosearch.mode.value;
	args.transactions = [];
	
	/*if( (!args.to || !args.from)){
		args.errorMsg = dw.web.Resource.msg('search.invalidFilters', 'backoffice', null);
		return renderSearch(args);
	}*/
	
	/*if(!args.transactionid){
		args.errorMsg = dw.web.Resource.msg('search.invalidFilters', 'backoffice', null);
		return renderSearch(args);
	}*/
	
	var iter = null;
	if(args.transactionid) {
		var query = '(custom.xpay_transactionCode LIKE {0} OR orderNo LIKE {0}) AND custom.xpay_transactionCode != null';
		iter = dw.order.OrderMgr.searchOrders(
			query, 
			'orderNo ASC', 
			args.transactionid);
		}
	else {
		var query = 'custom.xpay_transactionCode != null';
		iter = dw.order.OrderMgr.searchOrders(
			query, 
			'creationDate DESC', 
			'');
	}
			
	var numResults = 0;
	if(iter.getCount() > 0) {		
		while(iter.hasNext() && numResults < MAX_QUERY_RESULTS) {
			var order = iter.next();
			
			var paymentInstruments = order.getPaymentInstruments();
            pisIter = paymentInstruments.iterator();
            while(pisIter.hasNext()) {
                var pi : PaymentInstrument = pisIter.next();                

				if(!pi.paymentTransaction.transactionID) {
					continue;
				}

				//if mode is "Capture" removes any transaction with immediate accounting type
				//this filter could not be placed in the query
				if(args.mode != MODE_REIMBURSE 
					&& pi.paymentTransaction.custom.xpay_tcontab != null
					&& pi.paymentTransaction.custom.xpay_tcontab.value != 'D') {
					continue;
				}
				
				var transaction = {
					alias : pi.paymentTransaction.custom.xpay_alias,
					codiceTransazione: pi.paymentTransaction.transactionID,
					importo: pi.paymentTransaction.amount,
					divisa: 'EUR',
					dataTransazione: pi.paymentTransaction.custom.xpay_data,
					//brand: pi.paymentTransaction.,
					//stato: pi.paymentTransaction.,
					orderNumber: order.orderNo,				
					orderToken: order.orderToken,
					orderStatus: order.status.displayValue,								
					//orderLink: dw.web.URLUtils.url('ViewOrder_52-FindByNumber', 'OrderNo', order.orderNo);
					tcontabLabel : pi.paymentTransaction.custom.xpay_tcontab.displayValue,
					tcontabValue : pi.paymentTransaction.custom.xpay_tcontab.value,
					actionLink: dw.web.URLUtils.url('NexiPayTransactionMng-SelectTransaction',  
							'TransactionId', pi.paymentTransaction.transactionID,
							'orderNum', order.orderNo,
							'TopUrl', session.forms.bosearch.bc_root_link.value,
							'MainMenuName', session.forms.bosearch.bc_root_name.value,
							'mode', args.mode,
							'alias', pi.paymentTransaction.custom.xpay_alias),
						
				};
				args.transactions.push(transaction);
				numResults++;			
			}			
		}
	}
	iter.close();
	
	return renderSearch(args);
}

/**
 * Select a single transaction controller's logic
 * @param args <Object>
 * 				- Some additional parameters that may be passed from outside (e.g. "successMsg")
 * @returns
 */
function selectTransaction(args){

	args = args || {};

	var mainMenuName 	= request.httpParameterMap.MainMenuName.stringValue || session.forms.bosearch.bc_root_name.value;
	var topUrl			= request.httpParameterMap.TopUrl.stringValue || session.forms.bosearch.bc_root_link.value;
	var transactionId	= request.httpParameterMap.TransactionId.stringValue || session.forms.bosearch.transactionid.value;
	var editAmount 		= request.httpParameterMap.EditMode.stringValue === EDIT_AMOUNT;
	var mode 			= request.httpParameterMap.mode.stringValue || session.forms.bosearch.mode.value || MODE_REIMBURSE;
	var orderNum 		= request.httpParameterMap.orderNum.value || session.forms.bosearch.orderNum.value;
    
	session.forms.bosearch.bc_root_name.value = mainMenuName;
    session.forms.bosearch.bc_root_link.value = topUrl;
	session.forms.bosearch.transactionid.value = transactionId;
	session.forms.bosearch.mode.value = mode;
	session.forms.bosearch.orderNum.value = orderNum;

	var alias = request.httpParameterMap.alias.stringValue || session.custom.alias;
	
	var errorMsg = '';
    var transaction = null;
    
    //no transaction id
    if(!transactionId){
    	return renderSearch({
    		topUrl: topUrl,
    		mainMenuName: mainMenuName,
			mode: mode,
			alias : alias
    	});
	}
	
	
    
	transaction  = getCachedTransaction(); 
	
	//if no transaction on session or current transaction in session has a different transaction id
    if(!transaction || transaction.codiceTransazione !== transactionId){    	
		
    	//query Nexi to get a given transaction
	    var result = getXPayTransaction(transactionId,alias);
		
		
	    if(result.esito !== 'OK'){
			
	    	//error retrieving the transaction
	    	errorMsg = result.errore.messaggio +' (XPay error code: '+result.errore.codice+')';
			logger.error('ERROR: '+errorMsg);
			
			session.forms.bosearch.transaction_cache.value = null;			
		} else {	    	
			//gets the transaction
			transaction = result.report[0];
	    	//get related order
	    	//var order = getOrderByTransactionId(transaction.codiceTransazione);
	    	var order = dw.order.OrderMgr.getOrder(orderNum);
			if(order){
				transaction.orderNumber = order.orderNo;
				transaction.orderToken = order.orderToken;
				//transaction.order = order.status;
				transaction.tcontabLabel = order.paymentTransaction.custom.xpay_tcontab.displayValue;
				transaction.tcontabValue = order.paymentTransaction.custom.xpay_tcontab.value;
							
				//transaction.orderLink = dw.web.URLUtils.url('ViewOrder_52-FindByNumber', 'OrderNo', order.orderNo);						
			} 
			
			var importoContabilizzato = 0; // totale delle operazioni di tipo CONTAB
			var importoStornato = 0;  // totale delle operazioni di tipo STORNO

			if(transaction.dettaglio){		
				
				if(transaction.dettaglio.length > 0) {
					transaction.nome = transaction.nome || transaction.dettaglio[0].nome;
					transaction.cognome = transaction.cognome || transaction.dettaglio[0].cognome;
					transaction.mail = transaction.mail || transaction.dettaglio[0].mail;
				}
				
				for(var i = 0; i < transaction.dettaglio.length; i++){
					var t = transaction.dettaglio[0]['operazioni'];
					for(var j = 0; j < t.length; j++){
						if(t[j].tipoOperazione == 'STORNO'){
							if(t[j].stato != 'Annullato'){
								importoStornato = importoStornato + parseInt(t[j].importo) / 100;
							}
						}else if (t[j].tipoOperazione == 'CONTAB.'){
							// CALCOLO IMPORTO CONTABILIZZATO
							// L’importo contabilizzato deve essere il totale delle operazioni di tipo CONTAB meno il totale delle operazioni di tipo STORNO. (FONTE NEXI)
							// Qui eseguo il totale delle operazioni di tipo CONTAB 					
							if(t[j].stato != 'Annullato'){
								importoContabilizzato = importoContabilizzato + parseInt(t[j].importo) / 100;
							}
						}
					}	
				}

				importoContabilizzato = (importoContabilizzato - importoStornato);
			}
			
			transaction.importoContabilizzato = importoContabilizzato;
			transaction.importoStornato = importoStornato;

			//session.forms.bosearch.amount.value = parseInt(transaction.importo || 0) / 100;

			// modifica per impostare il valore del campo editAmountLink			
			if(mode == 'MODE_REIMBURSE' || mode == 'Reimburse'){
				// rimoborso / storno
				session.forms.bosearch.amount.value = importoContabilizzato;				
			}else{
				// incasso
				session.forms.bosearch.amount.value = (parseInt(transaction.importo || 0) / 100 - importoContabilizzato);				
			}
			
			
			//used to cache transaction call to Nexi 
			//stringify needed to avoid compatibility mode issues 
			//(see https://documentation.b2c.commercecloud.salesforce.com/DOC2/index.jsp?topic=%2Fcom.demandware.dochelp%2FSiteDevelopment%2FCompatibilityModeConsiderations.html
			// and object serialization)
			session.forms.bosearch.transaction_cache.value = JSON.stringify(transaction);
	    }
    } else {		
    	//session.forms.bosearch.amount.value = parseInt(transaction.importo) / 100;
    	transaction = getCachedTransaction();
    }
    
    return renderTransactionDetails({
    	transaction: transaction, 
    	topUrl: topUrl,
    	mainMenuName: mainMenuName,
    	errorMsg: errorMsg,
    	editAmount: editAmount, 
    	transactionId: transactionId,
    	successMsg: args.successMsg,
		mode: mode,
		alias : alias
    });
    
}

/**
 * Issues the reimbursement controller's logic
 * @returns
 */
function doReimburse(){
	
	var mainMenuName 	= session.forms.bosearch.bc_root_name.value;
	var topUrl			= session.forms.bosearch.bc_root_link.value;
	var transactionId	= session.forms.bosearch.transactionid.value;
	var amount			= session.forms.bosearch.amount.value || 0;
	var transaction 	= getCachedTransaction();
	var mode			= session.forms.bosearch.mode.value || MODE_REIMBURSE;
	var errorMsg = null;
	var successMsg = null;

	var alias = request.httpParameterMap.alias.stringValue || session.custom.alias;

	session.custom.alias = alias;
	
	if(transaction){
		
		try{
			amount = (!amount) ? 0: Math.round(100*parseFloat(amount));
			if(amount <= 0){
				throw 'Invalid amount value: '+amount;
			}
		}catch(ex){
			//session.forms.bosearch.amount.value = 0;
			logger.error('ERROR (amount): '+ex);
			return renderTransactionDetails({
		    	transaction: transaction, 
		    	topUrl: topUrl,
		    	mainMenuName: mainMenuName,
		    	errorMsg: dw.web.Resource.msg('detail.invalidAmount', 'backoffice', null),
		    	editAmount: true, 
		    	transactionId: transactionId,
		    	mode: mode,
		    });
		}
			
		//call to nexi
		var result = doXPayReimburse({
			transactionId: transactionId,
			amount: amount,
			alias : alias
		});
		
		if(result.esito !== 'OK'){
			//result is KO
			errorMsg =  result.errore.messaggio +' ('+result.errore.codice+')';//dw.web.Resource.msg('detail.unexpectedError', 'backoffice', null);
			//editAmount = true;
			logger.error('ERROR: '+errorMsg);
			
			return renderTransactionDetails({
		    	transaction: transaction, 
		    	topUrl: topUrl,
		    	mainMenuName: mainMenuName,
		    	errorMsg: errorMsg,
		    	editAmount: true, 
		    	transactionId: transactionId,
		    	mode: mode
		    });
			
		}else{
			//succesfully issued a reimburse
			//refresh the transaction on session as we have just created a new entry on the operations history on Nexi side
			session.forms.bosearch.transaction_cache.value = null;
			
			return selectTransaction({
				successMsg: dw.web.Resource.msg('detail.reimburse.success', 'backoffice', null),
				mode: mode,
			});
		}
		
	}
	//no transaction on session, redirect to "search" page
	return response.redirect(dw.web.URLUtils.https('NexiPayTransactionMng-Start', 'mainmenuname', mainMenuName, 'TopUrl', topUrl, 'mode', mode));
}

/**
 * Issues the taking controller's logic
 * @returns
 */
function doCapture(){
	
	var mainMenuName 	= session.forms.bosearch.bc_root_name.value;
	var topUrl			= session.forms.bosearch.bc_root_link.value;
	var transactionId	= session.forms.bosearch.transactionid.value;
	var amount			= session.forms.bosearch.amount.value || 0;
	var transaction 	= getCachedTransaction();
	var mode			= session.forms.bosearch.mode.value || MODE_REIMBURSE;
	var errorMsg = null;
	var successMsg = null;

	var alias = request.httpParameterMap.alias.stringValue || session.custom.alias;

	session.custom.alias = alias;
	
	if(transaction){
		
		try{
			amount = (!amount) ? 0: Math.round(100*parseFloat(amount));
			if(amount <= 0){
				throw 'Invalid amount value: '+amount;
			}
		}catch(ex){
			//session.forms.bosearch.amount.value = 0;
			logger.error('ERROR (amount): '+ex);
			return renderTransactionDetails({
		    	transaction: transaction, 
		    	topUrl: topUrl,
		    	mainMenuName: mainMenuName,
		    	errorMsg: dw.web.Resource.msg('detail.invalidAmount', 'backoffice', null),
		    	editAmount: true, 
		    	transactionId: transactionId,
				mode: mode,
				alias : alias
		    });
		}
			
		//call to nexi
		var result = doXPayCapture({
			transactionId: transactionId,
			amount: amount,
			alias: alias
		});
		
		if(result.esito !== 'OK') {
			//result is KO
			errorMsg =  result.errore.messaggio +' ('+result.errore.codice+')';//dw.web.Resource.msg('detail.unexpectedError', 'backoffice', null);
			//editAmount = true;
			logger.error('ERROR: '+errorMsg);
			
			return renderTransactionDetails({
		    	transaction: transaction, 
		    	topUrl: topUrl,
		    	mainMenuName: mainMenuName,
		    	errorMsg: errorMsg,
		    	editAmount: true, 
		    	transactionId: transactionId,
		    	mode: mode,
				alias : alias
		    });
		}
		else {
			//succesfully issued a reimburse
			//refresh the transaction on session as we have just created a new entry on the operations history on Nexi side
			session.forms.bosearch.transaction_cache.value = null;
			
			return selectTransaction({
				successMsg: dw.web.Resource.msg('detail.taking.success', 'backoffice', null),
				mode: mode,
			});
		}
	}
	//no transaction on session, redirect to "search" page
	return response.redirect(dw.web.URLUtils.https('NexiPayTransactionMng-Start', 'mainmenuname', mainMenuName, 'TopUrl', topUrl, 'mode', mode));
}

/**
 * Initialize the search transaction page
 * @returns
 */
function start(pdict){
	session.forms.bosearch.transactionid.value = '';
	session.forms.bosearch.todate.value = '';
	session.forms.bosearch.fromdate.value = '';
	session.forms.bosearch.mode.value = request.httpParameterMap.mode.stringValue;	
	session.forms.bosearch.transaction_cache.value = null; //used to cache transaction call to Nexi
	
	var topUrl = dw.web.URLUtils.url('SiteNavigationBar-ShowMenuitemOverview', 'CurrentMenuItemId', request.httpParameterMap.CurrentMenuItemId);
	if(!request.httpParameterMap.CurrentMenuItemId.value){
		topUrl = request.httpParameterMap.TopUrl.value;
	}
    var mainMenuName = request.httpParameterMap.mainmenuname.stringValue;
    session.forms.bosearch.bc_root_name.value = mainMenuName;
    session.forms.bosearch.bc_root_link.value = topUrl;
    
	return renderSearch({
		topUrl: topUrl,
		mainMenuName: mainMenuName,
		mode: session.forms.bosearch.mode.value,
	});	
}

/**
 * Handles form actions (search and reimbursement actions)
 * @returns
 */
function handleForm() {
	
	session.forms.bosearch.transactionid.value = session.forms.bosearch.transactionid.value || '';
	session.forms.bosearch.todate.value = session.forms.bosearch.todate.value || '';
	session.forms.bosearch.fromdate.value = session.forms.bosearch.fromdate.value || '';
	session.forms.bosearch.mode.value = session.forms.bosearch.mode.value || request.httpParameterMap.mode.stringValue || MODE_REIMBURSE;
	
	var triggeredAction = request.triggeredFormAction;
	if (!empty(triggeredAction)) {
		if(triggeredAction.formId === 'search') {			
			return getTransactions();
		} else if(triggeredAction.formId === 'reimburse') {			
			return doReimburse();
		} else if(triggeredAction.formId === 'take'){			
			return doCapture();
		} else if(triggeredAction.formId === 'cancel') {			
			return selectTransaction();
		}
	}
	return start();
}

exports.Start 				= guard.ensure(['https'], start);
exports.HandleForm 			= guard.ensure(['https', 'post'], handleForm);
exports.SelectTransaction 	= guard.ensure(['https'], selectTransaction);

