'use strict';

/* API Includes */
var Cart = require('*/cartridge/scripts/models/CartModel');
var PaymentMgr = require('dw/order/PaymentMgr');
var Order = require('dw/order/Order');
var Transaction = require('dw/system/Transaction');
var Status = require('dw/system/Status');

/* Script Modules */
var XPayUtils = require('~/cartridge/scripts/util/XPayUtils');
var logger = dw.system.Logger.getLogger('nexi.xpay.int');

/**
 * Verifies a credit card against a valid card number and expiration date and possibly invalidates invalid form fields.
 * If the verification was successful a credit card payment instrument is created.
 * 
 * @see module:models/PaymentProcessorModel~PaymentProcessorModel/handle
 * 
 * @param {dw.order.Basket} args.cart The order to validate.
 * @param {String} args.paymentMethodID The ID of payment method to execute the validation logic for.
 */
function Handle(args) {

	logger.info('\nXPAY.Handle');
	
	try {
	    Transaction.wrap(function () {	    	
	        var cart = Cart.get(args.Basket);

	        cart.removeExistingPaymentInstruments(XPayUtils.XPAY_PAYINST);

			var paymentInstrument = cart.createPaymentInstrument(XPayUtils.XPAY_PAYINST, cart.getNonGiftCertificateAmount());
			paymentInstrument.paymentTransaction.custom.xpay_tcontab = XPayUtils.XPAY_TCONTAB;
		});
		
		logger.info('\nXPAY.Handle complete.');
	    return { success: true };
	}
    catch(ex) {
		if(ex.errMsg) logger.error('\nERROR: '+ex.errMsg);
		session.custom.error = ex.errMsg;		
		resetXpayFields();										
		return { error: true };
    }
}


/**
 * Authorizes a payment using a credit card. The payment is authorized by using the BASIC_CREDIT processor
 * only and setting the order no as the transaction ID. Customizations may use other processors and custom
 * logic to authorize credit card payment.
 * 
 * @see module:models/PaymentProcessorModel~PaymentProcessorModel/authorize
 * 
 * @param {dw.order.Order} Order The order to authorize.
 * @param {String} OrderNo The order number to authorize.
 * @param {dw.order.PaymentInstrument} PaymentInstrument The payment instrument to execute the authorization logic for.
 */
function Authorize(args) {
	
	logger.info('\nXPAY.Authorize');

	if(XPayUtils.XPAY_PAYMODE === XPayUtils.PM_XPAY_BUILD) {
		return Authorize_XPayBuild(args);
	}
	else if(XPayUtils.XPAY_PAYMODE === XPayUtils.PM_EASY_PAYMENT ||
	        XPayUtils.XPAY_PAYMODE === XPayUtils.PM_ONECLICK) {
		return Authorize_XPay(args);
	}
	else {
		logger.error('\nNo payment mode set. Check XPay preference.');
	}
}


/**
 * Authorizes a xpay build payment
 * 
 * @param {dw.order.Order} Order The order to authorize.
 * @param {String} OrderNo The order number to authorize.
 * @param {dw.order.PaymentInstrument} PaymentInstrument The payment instrument to execute the authorization logic for.
 */
function Authorize_XPayBuild(args) {
	
	logger.info('\nXPAY.Authorize_XPayBuild');
	
	var transactionCode = session.forms.billing.paymentMethods.xpay.xpayCodiceTransazione.value;
	var orderAmountInCents = session.forms.billing.paymentMethods.xpay.xpayImporto.value;
	var currency = XPayUtils.XPAY_CURRENCY;	
	var xpayEsito = session.forms.billing.paymentMethods.xpay.xpayEsito.value;
	var xpayIdOperazione = session.forms.billing.paymentMethods.xpay.xpayIdOperazione.value;
	var xpayNonce = session.forms.billing.paymentMethods.xpay.xpayNonce.value;
	var xpayTimeStamp = session.forms.billing.paymentMethods.xpay.xpayTimeStamp.value;	
	var numeroContratto = session.forms.billing.paymentMethods.xpay.numeroContratto.value;

	var xpayMac = session.forms.billing.paymentMethods.xpay.xpayMac.value;		
	
	try {						
				
		if(xpayEsito !== 'OK') {			
			logger.error('\nXPAY: Request nonce failed');

			//exit and skip transaction
	    	return {
				error: true
	    	}
		}
		
	    // calculate mac with the merchant secret key
	    var macCalculated = XPayUtils.sha1(
			'esito=' + xpayEsito + 
		    'idOperazione=' + xpayIdOperazione + 
		    'xpayNonce=' + xpayNonce + 
		    'timeStamp=' + xpayTimeStamp + 
		    XPayUtils.XPAY_TERM_WEB_SECRETKEY);		
		
		// ...and check if it matches with mac sent by Nexi
		if(macCalculated != xpayMac) {						
			logger.error('\nXPAY: Invalid response: MAC not matching');

			//exit and skip transaction
	    	return {
				error: true
	    	}
		}

		Transaction.begin();
	
		var httpResult : Result = null;
		var PayNonceSvc = require("~/cartridge/scripts/svc/PayNonceSvc.js");
		if(numeroContratto) {
			httpResult = PayNonceSvc.firstRecurPay({
				apiKey              : XPayUtils.XPAY_TERM_WEB_ALIAS,
				codiceTransazione   : transactionCode,
				importo             : orderAmountInCents,
				divisa              : currency,
				xpayNonce           : xpayNonce,
				numeroContratto     : numeroContratto,
				codiceGruppo        : XPayUtils.XPAY_TERM_WEB_GROUP,
				parametriAggiuntivi : {             
					TCONTAB  : XPayUtils.XPAY_TCONTAB,
					mail     : session.forms.billing.billingAddress.email.emailAddress.value
				}
			});	
		}
		else {
			httpResult = PayNonceSvc.payNonce({
				apiKey              : XPayUtils.XPAY_API_KEY,
				codiceTransazione   : transactionCode,
				importo             : orderAmountInCents,
				divisa              : currency,
				xpayNonce           : xpayNonce,
				parametriAggiuntivi : {             
					TCONTAB  : XPayUtils.XPAY_TCONTAB,
					mail     : session.forms.billing.billingAddress.email.emailAddress.value
				}
			});
		}
		logger.debug('PayNonceSvc.httpResult.status: '+httpResult.status);
		
		if(!httpResult.ok) {			
			throw { 
				errMsg: '['+httpResult.error+'] ' + httpResult.errorMessage
			}
		}
		
		//logger.debug('PayNonceSvc.httpResult.object:\n'+httpResult.object);
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('PayNonceSvc-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));
		
		if(jsonResponse.esito != 'OK') {		
			throw { 
				errMsg: '['+jsonResponse.errore.codice+'] ' + jsonResponse.errore.messaggio,
				jsonResponse: jsonResponse
			}
		}				
				
		//check mac response
		var macCalculated = XPayUtils.sha1(
			'esito=' + jsonResponse.esito + 
			'idOperazione=' + jsonResponse.idOperazione + 
			'timeStamp=' + jsonResponse.timeStamp + 
			XPayUtils.XPAY_TERM_WEB_SECRETKEY);			

		var isMacResponseValid = (jsonResponse.mac === macCalculated);
		
		var order = args.Order;
		order.custom.xpay_nonce = xpayNonce;
		order.custom.xpay_transactionCode = transactionCode;

		if(XPayUtils.isRecurringEnabled() && numeroContratto) { //check if recurrings data model is ok		
			order.custom.xpay_recurContractNum = numeroContratto;
		}

		var orderNo = args.OrderNo;
		var paymentInstrument = args.PaymentInstrument;
		var paymentProcessor = PaymentMgr.getPaymentMethod(paymentInstrument.getPaymentMethod()).getPaymentProcessor();
		paymentInstrument.paymentTransaction.transactionID = transactionCode;
		paymentInstrument.paymentTransaction.paymentProcessor = paymentProcessor;
		
		paymentInstrument.paymentTransaction.custom.xpay_esito = jsonResponse.esito;
		paymentInstrument.paymentTransaction.custom.xpay_idOperazione = jsonResponse.idOperazione;
		paymentInstrument.paymentTransaction.custom.xpay_codiceAutorizzazione = jsonResponse.codiceAutorizzazione;
		paymentInstrument.paymentTransaction.custom.xpay_codiceConvenzione = jsonResponse.codiceConvenzione;
		paymentInstrument.paymentTransaction.custom.xpay_data = jsonResponse.data;
		paymentInstrument.paymentTransaction.custom.xpay_ora = jsonResponse.ora;
		paymentInstrument.paymentTransaction.custom.xpay_nazione = jsonResponse.nazione;
		paymentInstrument.paymentTransaction.custom.xpay_regione = jsonResponse.regione;
        paymentInstrument.paymentTransaction.custom.xpay_tipoProdotto = jsonResponse.tipoProdotto;
		paymentInstrument.paymentTransaction.custom.xpay_tipoTransazione = jsonResponse.tipoTransazione;
		paymentInstrument.paymentTransaction.custom.xpay_timeStamp = jsonResponse.timeStamp;		
		paymentInstrument.paymentTransaction.custom.xpay_mac = jsonResponse.mac;
		paymentInstrument.paymentTransaction.custom.xpay_ppo = jsonResponse.ppo;
		paymentInstrument.paymentTransaction.custom.xpay_tcontab = XPayUtils.XPAY_TCONTAB;
		paymentInstrument.paymentTransaction.custom.xpay_alias = XPayUtils.XPAY_TERM_WEB_ALIAS;
			
		if(!isMacResponseValid) {
			// if response is OK but mac is not valid, we do not mark the order as paid
			var macErrMsg = "Invalid pay nonce response: MAC not matching";
			paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore = macErrMsg;
			logger.error(macErrMsg);
		}
	}
	catch(ex) {
		if(ex.errMsg) logger.error('\nERROR: '+ex.errMsg);
		
		Transaction.rollback();
		
		Transaction.wrap(function () {
			
			session.custom.error = ex.errMsg;
			resetXpayFields();
			
			var order = args.Order;
			order.custom.xpay_nonce = xpayNonce;
			order.custom.xpay_transactionCode = transactionCode;

			var paymentInstrument = args.PaymentInstrument;
		    var paymentProcessor = PaymentMgr.getPaymentMethod(paymentInstrument.getPaymentMethod()).getPaymentProcessor();
		    paymentInstrument.paymentTransaction.paymentProcessor = paymentProcessor;

		    var jsonResponse = ex.jsonResponse;
		    if(jsonResponse) {
		    	paymentInstrument.paymentTransaction.custom.xpay_esito = jsonResponse.esito;
		    	paymentInstrument.paymentTransaction.custom.xpay_idOperazione = jsonResponse.idOperazione;
		    	paymentInstrument.paymentTransaction.custom.xpay_timeStamp = jsonResponse.timeStamp;
		    	paymentInstrument.paymentTransaction.custom.xpay_codiceErrore = jsonResponse.errore.codice;
		    	paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore = jsonResponse.errore.messaggio;
		    }
		});
				
		return {
	    	error: true,
	    	PlaceOrderError: new Status(Status.ERROR, 'xpay.placeorder_failed') 
			/* il messaggio di errore nell'implementazione di default di SJ è predefinito nella proprietà 'confirm.error.technical' 
			   e mostrato in summary.isml */
	    }
	}

	Transaction.commit();
	
	logger.info('\nXPAY.Authorize complete.');
	return { authorized: true };

}


/**
 * Authorizes a generic XPay payment
 * 
 * @param {dw.order.Order} Order The order to authorize.
 * @param {String} OrderNo The order number to authorize.
 * @param {dw.order.PaymentInstrument} PaymentInstrument The payment instrument to execute the authorization logic for.
 */
function Authorize_XPay(args) {
	
	logger.info('\nXPAY.Authorize_XPay');
	
	var orderNo = args.OrderNo; 
	//logger.debug('\nXPAY-orderNo: '+orderNo);

	var transactionCode = session.custom.codTrans;
	var result = session.custom.esito;
	var resultCode = session.custom.codiceEsito;	
	var orderAmountInCents = session.custom.importo;
	var currency = session.custom.divisa;	
	var transacDate = session.custom.data;	
	var transacTime = session.custom.orario;	
	var authCode = session.custom.codAut;	
	var mac = session.custom.mac;		
	var alias = session.custom.alias || XPayUtils.XPAY_TERM_WEB_ALIAS;

	//NOP> session.custom.importo;
	//NOP> session.custom.divisa;
	//OK> session.custom.codTrans;
	var brand = session.custom.brand;		
	//OK> session.custom.data;
	//OK> session.custom.orario;	
	var codAut = session.custom.codAut;
	var pan = session.custom.pan;
	var panExpiry =  session.custom.scadenza_pan;
	var cardRegion = session.custom.regione;
	var cardCountry = session.custom.nazionalita;
	var description = session.custom.descrizione;
	var languageId = session.custom.languageId;
	var transacPayType = session.custom.tipoTransazione;
	var cardType = session.custom.tipoProdotto;
	var name = session.custom.nome;
	var surname = session.custom.cognome;
	var transacMsg = session.custom.messaggio;
	var mail = session.custom.mail;
	var hash = session.custom.hash;
	var infoc = session.custom.infoc;
	var infob = session.custom.infob;
	var codConv = session.custom.codiceConvenzione;

	try {	
		Transaction.begin();

		var order = args.Order;
		order.custom.xpay_transactionCode = transactionCode;

		var orderNo = args.OrderNo;
		var paymentInstrument = args.PaymentInstrument;
		var paymentProcessor = PaymentMgr.getPaymentMethod(paymentInstrument.getPaymentMethod()).getPaymentProcessor();
		paymentInstrument.paymentTransaction.transactionID = transactionCode;
		paymentInstrument.paymentTransaction.paymentProcessor = paymentProcessor;

		//transaction data
		paymentInstrument.paymentTransaction.custom.xpay_alias = alias;
		paymentInstrument.paymentTransaction.custom.xpay_esito = result;
		paymentInstrument.paymentTransaction.custom.xpay_codiceEsito = resultCode;
		paymentInstrument.paymentTransaction.custom.xpay_codiceAutorizzazione = codAut;
		paymentInstrument.paymentTransaction.custom.xpay_codiceConvenzione = codConv;
		paymentInstrument.paymentTransaction.custom.xpay_data = transacDate;
		paymentInstrument.paymentTransaction.custom.xpay_ora = transacTime;
		paymentInstrument.paymentTransaction.custom.xpay_nazione = cardCountry;
		paymentInstrument.paymentTransaction.custom.xpay_descrizione = description;
		paymentInstrument.paymentTransaction.custom.xpay_languageId = languageId;
		paymentInstrument.paymentTransaction.custom.xpay_regione = cardRegion;
		paymentInstrument.paymentTransaction.custom.xpay_tipoTransazione = transacPayType;
		paymentInstrument.paymentTransaction.custom.xpay_tipoProdotto = cardType;
		paymentInstrument.paymentTransaction.custom.xpay_messaggio = transacMsg;
		paymentInstrument.paymentTransaction.custom.xpay_mail = mail;
		paymentInstrument.paymentTransaction.custom.xpay_hash = hash;
		paymentInstrument.paymentTransaction.custom.xpay_infoc = infoc;
		paymentInstrument.paymentTransaction.custom.xpay_infob = infob;
		paymentInstrument.paymentTransaction.custom.xpay_mac = mac;
		paymentInstrument.paymentTransaction.custom.xpay_tcontab = XPayUtils.XPAY_TCONTAB;

		//card data
		paymentInstrument.creditCardHolder = name + ' ' + surname;
        paymentInstrument.creditCardNumber = pan;
        paymentInstrument.creditCardType = brand;
        paymentInstrument.creditCardExpirationYear = panExpiry ? panExpiry.substring(0,4) : '';
		paymentInstrument.creditCardExpirationMonth = panExpiry ? panExpiry.substring(4,6) : '';
		
		//customer data
		var profile = customer.getProfile();
		if(profile) {
			profile.custom.xpay_contractNumber = session.custom.num_contratto || profile.custom.xpay_contractNumber;
		}
	
		var terminal = XPayUtils.getTermByAlias(alias);

		// calculate mac with the merchant secret key
		var macCalculated = XPayUtils.sha1(
			'codTrans=' + transactionCode + 
			'esito=' + result + 
			'importo=' + orderAmountInCents + 
			'divisa=' + currency + 
			'data=' + transacDate + 
			'orario=' + transacTime + 
			'codAut=' + authCode + 
			terminal.secretkey);			

		// ...and check if it matches with mac sent by Nexi
		if(macCalculated != mac) {

			var macErrMsg = "Invalid response: MAC not matching";
			paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore = macErrMsg;
			logger.error('\nXPAY: '+ macErrMsg);			
			throw { errMsg: macErrMsg };
		}	
	}
	catch(ex) {
		if(ex.errMsg) logger.error('\nERROR: '+ex.errMsg);
		
		Transaction.rollback();
		
		Transaction.wrap(function () {
			
			session.custom.error = ex.errMsg;
			
			var order = args.Order;
			order.custom.xpay_transactionCode = transactionCode || '';

			var paymentInstrument = args.PaymentInstrument;
		    var paymentProcessor = PaymentMgr.getPaymentMethod(paymentInstrument.getPaymentMethod()).getPaymentProcessor();
		    paymentInstrument.paymentTransaction.paymentProcessor = paymentProcessor;

			paymentInstrument.paymentTransaction.custom.xpay_esito = result || '';
			paymentInstrument.paymentTransaction.custom.xpay_codiceErrore = resultCode || '';
			paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore = ex.errMsg || '';
		});
				
		return { error: true };
	}
			
	Transaction.commit();
		
	logger.info('XPAY.Authorize_XPay complete - result: {0} - resultCode: {1}', result, resultCode);
	
	if(result !== 'OK') {		
		return { error: true };
	}
	else
		return { authorized: true };
}

/**
 * reset xpay data stored in the billing form
 */
function resetXpayFields() {	
	session.forms.billing.paymentMethods.xpay.clearFormElement();
}


/*
 * Module exports
 */
exports.Handle = Handle;
exports.Authorize = Authorize;
